// Author : Pierre Vuillemin (2016)
// License : CECILL

function problem = sopi_addConstraints(problem, cList)
   problem.cList = cList;
   for c = cList
      if and(c.isLinear) then
         problem              = sopi_addLinearConstraint(problem, c)
         if and(c.isBound) then
            problem.cType($+1) = 0
         else
            problem.cType($+1) = 1
         end
      else
         if c.type == "ineq"
            problem.c($+1) = c
         else
            problem.ceq($+1) = c
         end
         problem.cType($+1) = 42
//         error("Only linear constraints are handled in this version")
      end
   end
endfunction

function problem = sopi_addLinearConstraint(problem, c)
   var = c.lhs
   b   = sopi_extractConstantPart(var)
   out = sopi_extractLinearPart(var)
   //
   isBound = %F;
   if length(out) == 1 then
      // Only one variable involved, it may be a bound
      L = out(1)(1)
      R = out(1)(2)
      F = R' .*. L
      v = out(1)(3)
      isBound = size(v,2) == 1 & and(abs(F) == eye(length(F),length(F)))
   end
   //
   if isBound then
      idx   = sopi_getVarIdxInProblem(problem, v)
      if sign(F(1,1)) < 0 then
         lb          = problem.lb
         lb(idx)     = -b
         problem.lb  = lb
      else
         ub          = problem.ub
         ub(idx)     = b
         problem.ub  = ub
      end
   else
      L  = out(1)(2)' .*. out(1)(1)
      nc = size(L,1)
      A  = zeros(nc, problem.nvar)
      B  = zeros(nc)
      for o = out
         v        = o(3)
         idx      = sopi_getVarIdxInProblem(problem, v)
         A(:,idx) = o(2)' .*. o(1)
         B        = -b
      end

      if c.type == "ineq"
         problem.A = [problem.A;A]
         problem.b = [problem.b;B]
      else
         problem.Aeq = [problem.Aeq;A]
         problem.beq = [problem.beq;B]
      end      

   end

endfunction
