// Author : Pierre Vuillemin (2016)
// License : CECILL

function out = %sopiProp_e(varargin)
   // Overloading of the extraction for a sopiProp
   //
   // Inputs
   // ------
   // 
   //
   // Returns
   // -------
   // The desired field or answer to the test.
   //
   p = varargin($)
   if length(varargin) == 2 & typeof(varargin(1)) == "string" then
      field             = varargin(1)
      validFieldNames   = fieldnames(p)
      if or(field == validFieldNames) then
         out = getfield(field, p)
      else
         out = sopi_performPropTest(p,field)
      end
   else
      error("Invalid extraction")
   end
endfunction

function out = sopi_performPropTest(p, field)
   out = %F
   select field
   case "isPoly"
      if or(p.sets == "rat")
         out = p.data.dorder == 0
      end
   case "isRat"
      if or(p.sets == "rat") 
         out = p.data.dorder >0
      end
   case "polyOrder"
      out = p.data.norder
   case "isDiff"
      out =  or(p.sets == "diff")
   case "isConvex"
      out =  or(p.sets == "convex")
   case "isConcave"
      out =  or(p.sets == "concave")
   case "isConstant"
      if p.isPoly
         out = p.polyOrder == 0
      end
   case "isLinear"
      if p.isPoly
         out = p.polyOrder == 1
      end
   case "isPWA"
      out = or(p.sets == "pwa")
   case "isNL"
      out = ~p.isPoly & ~p.isRat & ~p.isConvex & ~p.isPWA
   case "sign"
      out = p.data.sign
   else
      error(sprintf("Invalid field %s", field))
   end
endfunction
