// Author : Pierre Vuillemin (2016)
// License : CECILL

// sopiVar + sopiVar ...........................................................
function newVar = %sopiVar_a_sopiVar(var1,var2)
   sopi_print(5,"Performing sopiVar + sopiVar.\n")
   newSize  = sopi_checkSizesCoherence(var1,var2,"sum");
   if var1.isScalar & ~var2.isScalar then
      var1 = ones(newSize(1),newSize(2)) * var1;
   elseif var2.isScalar & ~var1.isScalar  then
      var2 = ones(newSize(1),newSize(2)) * var2;
   end
   newVar         = sopi_newVar("size"    ,      newSize,...
                                "operator",        "sum",...
                                "args"    ,  list(var1,var2))
   newVar.prop    = sopi_addProp(var1.prop,var2.prop)
endfunction


// sopi_addProp ................................................................
// applies, element-wise, the additions rules of two sets of properties.
function P = sopi_addProp(P1,P2)
   SETS     = "sets"
   CONVEX   = "convex"
   CONCAVE  = "concave"
   DIFF     = "diff"
   RAT      = "rat"
   PWA      = "PWA"
   m = size(P1)
   n = size(P1(1))
   P = sopi_newMatProp([m,n])
   for i = 1:m
      for j = 1:n
         P(i)(j) = sopi_additionRules(P1(i)(j),P2(i)(j))
      end
   end
endfunction

// sopi_additionRules ..........................................................
// returns a set of properties obtained after the addition of two sets of 
// properties corresponding to some sopiVar. For instance, the sum of two
// convex functions is also convex, etc.
function p = sopi_additionRules(p1, p2)
   p = sopi_newProp()
   if p1.isConvex & p2.isConvex then
      p.sets         = [p.sets;CONVEX]
   end
   if p1.isConcave & p2.isConcave then
      p.sets         = [p.sets;CONCAVE]
   end
   if p1.isRat & p2.isRat  | p1.isPoly & p2.isPoly then
      p.sets         = [p.sets;RAT]
      p.data.dorder  = max(p1.data.dorder,p2.data.dorder)
      p.data.norder  = max(p1.data.norder + p2.data.dorder, p1.data.dorder + p2.data.norder)
   end
   if p1.isPWA & p2.isPWA then
      p.sets         = [p.sets; PWA]
   end
   if p1.isDiff & p2.isDiff then
      p.sets         = [p.sets;DIFF]
   end
endfunction
