// Copyright 2015 - Pierre Vuillemin
// License : CECILL

// sopiVar * sopiVar ...........................................................
function newVar = %sopiVar_m_sopiVar(var1, var2)
   newSize           = sopi_checkSizesCoherence(var1, var2, "times")
   newVar            = sopi_newVar()
   newVar.size       = newSize
   newVar.operator   = "times"
   newVar.args       = list(var1,var2)
   newVar.prop       = sopi_mulProp(var1.prop, var2.prop)
endfunction

// Author : Pierre Vuillemin (2016)
// License : CECILL

// sopi_mulProp ................................................................
// returns a matrix of sets of properties of suitable size corresponding to the
// properties of the sopiVar obtained after matrix multiplication of two sopiVar.
function P = sopi_mulProp(P1,P2)
   SETS     = "sets"
   CONVEX   = "convex"
   CONCAVE  = "concave"
   DIFF     = "diff"
   POLY     = "poly"
   RAT      = "rat"
   m = size(P1)
   n = size(P1(1))
   o = size(P2(1))
   P = sopi_newMatProp([m,o])
   for i = 1:m
      for j = 1:o
         for k = 1:n
            newP = sopi_multiplicationRules(P1(i)(k),P2(k)(j))
            if k~=1
               P(i)(j) = sopi_additionRules(P(i)(j), newP)
            else
               P(i)(j) = newP
            end
         end
      end
   end
endfunction

// sopi_multiplicationRules ....................................................
// returns a set of properties obtained after the multiplication of two variables
// with some given properties. For instance, a negative constant times a convex 
// function is concave, etc.
// Obviously, those are note exhaustives.
function p = sopi_multiplicationRules(p1,p2)
   p = sopi_newProp()
   if p1.isConvex & p2.isConstant then
      if p2.sign > 0 
         p.sets = [p.sets;CONVEX]
      else
         p.sets = [p.sets;CONCAVE]
      end
   elseif p2.isConvex & p1.isConstant then
      if p1.sign > 0 
         p.sets = [p.sets;CONVEX]
      else
         p.sets = [p.sets;CONCAVE]
      end
   elseif p1.isConcave & p2.isConstant then
      if p2.sign > 0 then
         p.sets = [p.sets; CONCAVE]
      else
         p.sets = [p.sets;CONVEX]
      end
   elseif p2.isConcave & p1.isConstant then
      if p1.sign > 0 then
         p.sets = [p.sets;CONCAVE]
      else
         p.sets = [p.sets;CONVEX]
      end
   end
   if p1.isRat & p2.isRat | p1.isPoly & p2.isPoly then
      p.sets         = [p.sets;RAT]
      p.data.dorder  = p1.data.dorder + p2.data.dorder
      p.data.norder  = p1.data.norder + p2.data.norder
      if p.isPoly & p.polyOrder <= 1
         p.sets = [p.sets;CONVEX;CONCAVE]
      end
   end
   
   if p1.isDiff & p2.isDiff then
      p.sets         = [p.sets;DIFF]
   end
endfunction
