// Author : Pierre Vuillemin (2016)
// License : CECILL

function out = sopi_extractConstantPart(var)
   // Extracts the constant element of a variable.
   //
   // Input
   // -----
   // var (sopi_Var): the variable to analyse.
   // 
   // Returns
   // -------
   // A matrix.
   //
   // Example
   // -------
   // sopi_begin()
   // x        = sopi_var(3,3);
   // A        = rand(3,3);
   // B        = rand(3,3);
   // z        = A * x + B;
   // out      = sopi_extractConstantPart(z);
   // out == B
   //   
   if var.isElementary then
      out = zeros(size(var,1),size(var,2))
   else
      select var.operator
      case "constant"
         out = var.args(1)
      case "sum"
         out = sopi_extractConstantPart(var.args(1)) + sopi_extractConstantPart(var.args(2))
      case "times"
         out = sopi_extractConstantPart(var.args(1)) * sopi_extractConstantPart(var.args(2))
      case "transpose"
         out = sopi_extractConstantPart(var.args(1))
      else
         // TODO: implement other cases
         error("not yet")
      end
   end
endfunction

