// Author : Pierre Vuillemin (2016)
// License : CECILL

function sopi_begin(verbosity)
   // Initialises a storage variable for sopi.
   //
   // Calling Sequence
   // sopi_begin()
   // sopi_begin(verbosity)
   // 
   // Parameters
   // verbosity: the verbosity level of sopi (default is 0)
   //
   // Description
   // This routine creates a global variable sopiData in which some internal elements
   // required for sopi to work are stored.
   // This routine must be called prior to any sopi-related operation or they will
   // not work.
   // If this variable is altered, unexpected things will happen.
   //
   // In addition, it also enables to set global properties for sopi.
   //
   // See also
   // sopi_end()
   if ~isglobal("sopiData") then
      global sopiData
      sopiData             = mlist(["sopiNameSpace","varId","cstId","verbosity"]);
      sopiData.varId       = 0;
      sopiData.cstId       = 0;
      if ~exists("verbosity","local") then
         // quiet by default
         verbosity = -1
      end
      sopiData.verbosity   = verbosity; // quiet by default
      sopi_print(0,"Sopi verbosity set to %d.\n",sopiData.verbosity)      
   else
      sopi_print(-1,"sopiData already exists.")
   end
endfunction
