// Author : Pierre Vuillemin (2016)
// License : CECILL

// sopi_checkSizesCoherence ....................................................
// tests the coherence given two elements e1 and e2 and an operation. It returns
// the resulting size.
// /!\ do not change isscalar to e1.isScalar because matrices can be given in
// arguments
function newSize = sopi_checkSizesCoherence(e1,e2,operation)
   // Checks the coherence of an operation given two elements
   //
   // Inputs
   // ------
   // e1 (sopiVar)
   // e2 (sopiVar)
   // operation (string) : the operation performed between the sopiVar 
   //
   // Returns
   // -------
   // If the operation is not coherent, an error is raised. Otherwise, the size
   // of the resulting variable is returned.
   //
   mn1 = size(e1);
   mn2 = size(e2);
   select operation
   case "times"
      ok      = isscalar(e1) | isscalar(e2) | mn1(2) == mn2(1);
      if isscalar(e1) then
         newSize = mn2;
      elseif isscalar(e2) then
         newSize = mn1;
      else
         newSize = [mn1(1),mn2(2)];
      end
   case "sum"
      ok      = isscalar(e1) | isscalar(e2) | and(mn1 == mn2);
      if isscalar(e1) then
         newSize = mn2;
      elseif isscalar(e2) then
         newSize = mn1;
      else
         newSize = mn1;
      end
   case "constraint"
      ok       = and(mn1 == mn2);
      newSize = mn1;
   case "vercat"
      ok       = mn1(2) == mn2(2);
      newSize  = [mn1(1) + mn2(1), mn1(2)];
   case "horcat"
      ok       = mn1(1) == mn2(1);
      newSize  = [mn1(1), mn1(2) + mn2(2)];
   end

   if ~ok then
      error("Incompatible sizes : "+string(mn1(1))+"x"+string(mn1(2))+ " " + operation + " " + string(mn2(1))+"x"+string(mn2(2)))
   end
endfunction
