// Author : Pierre Vuillemin (2016)
// License : CECILL

// sopi_print ..................................................................
// prints a given string if the verbose mode is activated
function sopi_print(printLvl,strList,varargin)
   // Print a given string if its level is below the verbosity set with 
   // sopi_begin().
   //
   // Inputs
   // ------
   // printLvl (int): the importance of the message (must be lower than the 
   //    verbosity level to be displayed)
   // strList (string or list): the message together with some tags (first the tags
   //    then the message)
   // varargin (list): contains eventual data to be displayed
   //
   // Examples
   // --------
   // sopi_begin(verbosity = 1)
   // sopi_print(2,"Should not be displayed")
   // sopi_print(1,"Should be displayed")
   // sopi_print(1,["tag1", "Message"])
   // sopi_print(1, "Message with data: %d", 3)
   // sopi_print(1,["tag1", "Message with tag and data: %d"], 3)
   //
   [message, tagStr] = sopi_extractTags(strList);
   message           = " " + message;
   if sopi_verbosity()>= printLvl then
      if printLvl == -1 then
         tagStr = tagStr + "[WARNING]";
      end
      mprintf(tagStr + message, varargin(:));
   end
endfunction

// sopi_extractTags ............................................................
// separate the message from the tags given a string or matrix of strings.
function [message, tagStr] = sopi_extractTags(strList)
   strList = ["sopi",strList];
   if  size(strList,2) > 1 then
      message     = strList($);
      tags        = strList(1:$-1);
      tagStr      = "[" + strcat(tags,"][") + "]";
   else
      message  = strList;
      tagStr   = [];
   end
endfunction
